@@ -20,7 +20,6 @@ import cStringIO
 import gc
 import glob
 import os
-import pdb
 import re
 import sys
 import tempfile
@@ -45,10 +44,12 @@ import zope.testing.testrunner.garbagecollection
 import zope.testing.testrunner.listing
 import zope.testing.testrunner.statistics
 import zope.testing.testrunner.subprocess
-
+import zope.testing.testrunner.interfaces
+import zope.testing.testrunner.debug
 
 PYREFCOUNT_PATTERN = re.compile('\[[0-9]+ refs\]')
 
+is_jython = sys.platform.startswith('java')
 
 class SubprocessError(Exception):
     """An error occurred when running a subprocess
@@ -66,14 +67,6 @@ class CanNotTearDown(Exception):
     "Couldn't tear down a test"
 
 
-class EndRun(Exception):
-    """Indicate that the existing run call should stop
-
-    Used to prevent additional test output after post-mortem debugging.
-
-    """
-
-
 class Runner(object):
     """The test runner.
 
@@ -186,8 +179,13 @@ class Runner(object):
         self.features.append(zope.testing.testrunner.coverage.Coverage(self))
         self.features.append(zope.testing.testrunner.doctest.DocTest(self))
         self.features.append(zope.testing.testrunner.profiling.Profiling(self))
-        self.features.append(zope.testing.testrunner.garbagecollection.Threshold(self))
-        self.features.append(zope.testing.testrunner.garbagecollection.Debug(self))
+        if is_jython:
+            # Jython GC support is not yet implemented
+            pass
+        else:
+            self.features.append(zope.testing.testrunner.garbagecollection.Threshold(self))
+            self.features.append(zope.testing.testrunner.garbagecollection.Debug(self))
+
         self.features.append(zope.testing.testrunner.find.Find(self))
         self.features.append(zope.testing.testrunner.subprocess.SubProcess(self))
         self.features.append(zope.testing.testrunner.filter.Filter(self))
@@ -214,7 +212,7 @@ class Runner(object):
             try:
                 self.ran += run_layer(self.options, layer_name, layer, tests,
                                       setup_layers, self.failures, self.errors)
-            except EndRun:
+            except zope.testing.testrunner.interfaces.EndRun:
                 self.failed = True
                 return
             except CanNotTearDown:
@@ -248,8 +246,12 @@ def run_tests(options, tests, name, failures, errors):
 
     output = options.output
 
-    gc.collect()
-    lgarbage = len(gc.garbage)
+    if is_jython:
+        # Jython has no GC suppport - set count to 0
+        lgarbage = 0
+    else:
+        gc.collect()
+        lgarbage = len(gc.garbage)
 
     sumrc = 0
     if options.report_refcounts:
@@ -309,10 +311,13 @@ def run_tests(options, tests, name, failures, errors):
         output.summary(result.testsRun, len(result.failures), len(result.errors), t)
         ran = result.testsRun
 
-        gc.collect()
-        if len(gc.garbage) > lgarbage:
-            output.garbage(gc.garbage[lgarbage:])
-            lgarbage = len(gc.garbage)
+        if is_jython:
+            lgarbage = 0
+        else:
+            gc.collect()
+            if len(gc.garbage) > lgarbage:
+                output.garbage(gc.garbage[lgarbage:])
+                lgarbage = len(gc.garbage)
 
         if options.report_refcounts:
 
@@ -353,7 +358,7 @@ def run_layer(options, layer_name, layer, tests, setup_layers,
 
     try:
         setup_layer(options, layer, setup_layers)
-    except EndRun:
+    except zope.testing.testrunner.interfaces.EndRun:
         raise
     except Exception:
         f = cStringIO.StringIO()
@@ -537,7 +542,8 @@ def setup_layer(options, layer, setup_layers):
                             % options.resume_layer)
                         raise
                     else:
-                        post_mortem(sys.exc_info())
+                        zope.testing.testrunner.debug.post_mortem(
+                            sys.exc_info())
                 else:
                     raise
 
@@ -608,7 +614,7 @@ class TestResult(unittest.TestResult):
                                                       " when running a layer"
                                                       " as a subprocess!")
             else:
-                post_mortem(exc_info)
+                zope.testing.testrunner.debug.post_mortem(exc_info)
 
     def addFailure(self, test, exc_info):
         self.options.output.test_failure(test, time.time() - self._start_time,
@@ -619,16 +625,19 @@ class TestResult(unittest.TestResult):
         if self.options.post_mortem:
             # XXX: mgedmin: why isn't there a resume_layer check here like
             # in addError?
-            post_mortem(exc_info)
+            zope.testing.testrunner.debug.post_mortem(exc_info)
 
     def stopTest(self, test):
         self.testTearDown()
         self.options.output.stop_test(test)
 
-        if gc.garbage:
-            self.options.output.test_garbage(test, gc.garbage)
-            # TODO: Perhaps eat the garbage here, so that the garbage isn't
-            #       printed for every subsequent test.
+        if is_jython:
+            pass
+        else:
+            if gc.garbage:
+                self.options.output.test_garbage(test, gc.garbage)
+                # TODO: Perhaps eat the garbage here, so that the garbage isn't
+                #       printed for every subsequent test.
 
         # Did the test leave any new threads behind?
         new_threads = [t for t in threading.enumerate()
@@ -647,12 +656,20 @@ def layer_from_name(layer_name):
        to allow locating layers in cases where it would otherwise be
        impossible.
     """
-    global _layer_name_cache
     if _layer_name_cache.has_key(layer_name):
         return _layer_name_cache[layer_name]
     layer_names = layer_name.split('.')
     layer_module, module_layer_name = layer_names[:-1], layer_names[-1]
-    return getattr(import_name('.'.join(layer_module)), module_layer_name)
+    module_name = '.'.join(layer_module)
+    module = import_name(module_name)
+    try:
+        return getattr(module, module_layer_name)
+    except AttributeError, e:
+        # the default error is very uninformative:
+        #   AttributeError: 'module' object has no attribute 'DemoLayer'
+        # it doesn't say *which* module
+        raise AttributeError('module %r has no attribute %r'
+                             % (module_name, module_layer_name))
 
 
 def order_by_bases(layers):
@@ -682,42 +699,6 @@ def gather_layers(layer, result):
         gather_layers(b, result)
 
 
-def post_mortem(exc_info):
-    err = exc_info[1]
-    if isinstance(err, (doctest.UnexpectedException, doctest.DocTestFailure)):
-
-        if isinstance(err, doctest.UnexpectedException):
-            exc_info = err.exc_info
-
-            # Print out location info if the error was in a doctest
-            if exc_info[2].tb_frame.f_code.co_filename == '<string>':
-                print_doctest_location(err)
-
-        else:
-            print_doctest_location(err)
-            # Hm, we have a DocTestFailure exception.  We need to
-            # generate our own traceback
-            try:
-                exec ('raise ValueError'
-                      '("Expected and actual output are different")'
-                      ) in err.test.globs
-            except:
-                exc_info = sys.exc_info()
-
-    print "%s:" % (exc_info[0], )
-    print exc_info[1]
-    pdb.post_mortem(exc_info[2])
-    raise EndRun
-
-
-def print_doctest_location(err):
-    # This mimics pdb's output, which gives way cool results in emacs :)
-    filename = err.test.filename
-    if filename.endswith('.pyc'):
-        filename = filename[:-1]
-    print "> %s(%s)_()" % (filename, err.test.lineno+err.example.lineno+1)
-
-
 class FakeInputContinueGenerator:
 
     def readline(self):
